@@ -21,10 +21,10 @@ HQ = { # header quantities and corresponding lines
 
 @beartype
 def load_file(
-    qtynam: Union[List, Tuple],
+    qtynam: Union[list, tuple],
     impstm: str,
     **kwargs,
-) -> Tuple:
+) -> tuple:
     """
     Return the values of the quantities in qtynam from the output file.
 
@@ -41,31 +41,23 @@ def load_file(
         qtyval: quantity values in the order of qtynam
 
     The following quantities can be loaded:
+        'd' (Scalar): dislocation density [nm^-2]
+        'A' (ScalarListList): Fourier transform for each harmonic
         'n' (int): number of dislocations
         's' (int): size of the region of interest [nm]
         'g' (Vector): diffraction vector direction (hkl)
         'z' (Vector): direction of 'l' (line vector) [uvw]
         'b' (Vector): Burgers vector direction [uvw]
         'C' (Scalar): contrast factor [1]
-        'a' (Scalar): latice parameter [nm]
+        'a' (Scalar): lattice parameter [nm]
         'L' (ScalarList): Fourier variable [nm]
-        'A' (ScalarListList): Fourier amplitudes for each harmonic
-        'cos_AL' (ScalarList): cos of the first harmonic
-        'sin_AL' (ScalarList): sin of the first harmonic
-        'Cos_<j>AL' (ScalarList): cos of the <j>th harmonic
-        'Sin_<j>AL' (ScalarList): sin of the <j>th harmonic
-        'err_Cos' (ScalarList): cos error of the first harmonic
-        'err_Sin' (ScalarList): sin error of the first harmonic
-        'err_Cos_<j>AL' (ScalarList): cos error of the <j>th harmonic
-        'err_Sin_<j>AL' (ScalarList): sin error of the <j>th harmonic
         '<eps^2>' (ScalarList): mean square strain [1]
         'bad_points' (ScalarList): number of incorrect random points
     """
     # optional parameters
-    impdir = kwargs.pop('impdir', '') # import directory
-    impfmt = kwargs.pop('impfmt', 'dat') # import format
-    if len(kwargs)>0:
-        raise ValueError("wrong keywords: "+str(kwargs))
+    impdir = getkwa('impdir', kwargs, str, '')
+    impfmt = getkwa('impfmt', kwargs, str, 'dat')
+    endkwa(kwargs)
     # load file
     with open(os.path.join(impdir, impstm+'.'+impfmt), "r") as f:
         hv = [f.readline().strip("\n") for i in range(HL)] # header values
@@ -79,32 +71,43 @@ def load_file(
             i += 1 # go to next requested quantity
     qtyval = [] # values of the quantities in qtynam
     # sort values
-    for nam in qtynam:
+    def aux(nam: str):
         if nam in HQ: # the quantity is in the header
             v = np.array([eval(v) for v in hv[HQ[nam]].split('#')[0].split()])
             if len(v) == 1: # if the quantity is a scalar
                 v = v[0] # return a scalar
-            qtyval.append(v)
+            return v
         elif nam in tq: # the quantity is in the table
             j = tq.index(nam)
-            qtyval.append(np.array([eval(td[i][j]) for i in range(len(td))]))
+            return np.array([eval(td[i][j]) for i in range(len(td))])
         elif nam == 'A':
-            qtyval.append(
-                np.array([np.sqrt(
-                    np.array([eval(td[i][4*h+1]) for i in range(len(td))])**2
-                  + np.array([eval(td[i][4*h+3]) for i in range(len(td))])**2
-                ) for h in range((len(tq)-3)//4)])
+            a = (
+                np.array([
+                    np.array([eval(td[i][4*h+1]) for i in range(len(td))])
+                +1j*np.array([eval(td[i][4*h+3]) for i in range(len(td))])
+                for h in range((len(tq)-3)//4)])
             )
+            return a
+        elif nam =='d':
+            for stm in (impstm, impdir):
+                if 'rho' in stm and 'm-2' in stm:
+                    return eval(stm.split('rho')[1].split('m-2')[0])*1e-18
+            if 'Circle' in hv[HQ['s']]:
+                return aux('n')/2/np.pi/aux('s')**2
+            else:
+                return aux('n')/aux('s')**2
         else:
-            raise ValueError("unknown quantity: "+nam)
+            raise ValueError(f"unknown quantity: {nam}")
+    for nam in qtynam:
+        qtyval.append(aux(nam))
     return tuple(qtyval)
 
 @beartype
 def load_directory(
-    qtynam: Union[List, Tuple],
+    qtynam: Union[list, tuple],
     impstm: str,
     **kwargs,
-) -> Tuple:
+) -> tuple:
     """
     Average and return the values of qtynam over multiple files.
 
@@ -121,9 +124,8 @@ def load_directory(
         qtyval: averaged quantity values in the order of qtynam
     """
     # optional parameters
-    impdir = kwargs.pop('impdir', '') # import directory
-    if len(kwargs)>0:
-        raise ValueError("wrong keywords: "+str(kwargs))
+    impdir = getkwa('impdir', kwargs, str, '')
+    endkwa(kwargs)
     # load files
     dv = [] # values for each distribution file
     dir_stm = os.path.join(impdir, impstm) # files directory
@@ -139,10 +141,10 @@ def load_directory(
 
 @beartype
 def load(
-    qtynam: Union[List, Tuple],
+    qtynam: Union[list, tuple],
     impnam: str,
     **kwargs,
-) -> Tuple:
+) -> tuple:
     """
     Return the values of qtynam from a simulation output file or dir.
 
@@ -155,9 +157,8 @@ def load(
         qtyval: quantity values in the order of qtynam
     """
     # optional parameters
-    impdir = kwargs.pop('impdir', '') # import directory
-    if len(kwargs)>0:
-        raise ValueError("wrong keywords: "+str(kwargs))
+    impdir = getkwa('impdir', kwargs, str, '')
+    endkwa(kwargs)
     # load output
     outpth = os.path.join(impdir, impnam) # path to the output
     if os.path.isfile(outpth): # load the output file
@@ -166,4 +167,4 @@ def load(
     elif os.path.isdir(outpth): # load and average the output directory
         return load_directory(qtynam, impnam, impdir=impdir)
     else:
-        raise ValueError("nothing found at specified path: "+outpth)
+        raise ValueError(f"nothing found at specified path: {outpth}")
